class GuiActions:
    tab = "↔TAB"
    leave   = "Sair"
    edit = "Editar"
    interact  = "Interagir"
    principal = "Principal"
    limite = "Limite"
    github = "Github"
    help = "Ajuda"
    download = "Baixar"
    activate = "Ativar"
    navigate = "←↓→"
    progress = "Quanto"
    uncheck = "Desmarcar"
    search = "Pesquisar"
    self_grade = " Como "
    palette = "Paleta"
    all = "Todos"

class GuiKeys:
    left = "a"
    right = "d"
    down = "s"
    up = "w"

    palette = "p"
    images = "I"
    down_task = "b"
    activate = "\n"
    key_help = "?"
    inc_prog = ">"
    prog_plus2 = "."
    dec_prog = "<"
    prog_less2 = ","
    inc_self = "+"
    inc_grade2 = "="
    dec_self = "-"
    dec_grade2 = "_"
    set_root_dir = "D"
    set_lang = "L"
    github_open = "g"
    key_quit = "q"
    edit= "e"
    colors = "C"
    borders = "B"
    search = "/"
    graph = "G"
    diff = "m"
    interact = "i"
    test = "\n"
    main = "\t"
    quit   = "q"
    lock = "t"
    limite  = "l"
    expand = "]"
    collapse = "["